package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.dto.PlanDetailDTO;
import com.diagbot.dto.PlanInfoPageDTO;
import com.diagbot.dto.SysPlanInfoDTO;
import com.diagbot.entity.Plan;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.StatusEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.PlanServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.util.SysUserUtils;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.HospitalPlanCancelVO;
import com.diagbot.vo.HospitalPlanPageVO;
import com.diagbot.vo.HospitalPlanSaveVO;
import com.diagbot.vo.HospitalPlanVO;
import com.diagbot.vo.HospitalSetVO;
import com.diagbot.vo.PlanRevStopVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author wangfeng
 * @Description:
 * @date 2020-07-27 19:23
 */
@Component
public class PlanFacade extends PlanServiceImpl {

    @Autowired
    PlanDetailFacade sysSetFacade;

    public SysPlanInfoDTO getSysPlanInfoData(HospitalPlanVO hospitalPlanVO) {
        //查询该医院有没有相关方案配置， 要是没有取默认的
        SysPlanInfoDTO sysPlanInfoDTO = new SysPlanInfoDTO();
        QueryWrapper<Plan> sysPlanInfo = new QueryWrapper<>();
        sysPlanInfo.eq("is_deleted", IsDeleteEnum.N.getKey());
        sysPlanInfo.eq("hospital_id", hospitalPlanVO.getHospitalId());
        sysPlanInfo.eq(StringUtil.isNotBlank(hospitalPlanVO.getPlanCode()), "plan_code", hospitalPlanVO.getPlanCode());
        sysPlanInfo.eq("plan_status", StatusEnum.Enable.getKey());
        Plan sysPlanInfoData = getOne(sysPlanInfo, false);
        Long planId = -1L;
        Long hospitalId = hospitalPlanVO.getHospitalId();
        if (sysPlanInfoData != null) {
            BeanUtil.copyProperties(sysPlanInfoData, sysPlanInfoDTO);
            planId = sysPlanInfoDTO.getId();
        } else {
            QueryWrapper<Plan> sysPlandefault = new QueryWrapper<>();
            sysPlandefault.eq("is_deleted", IsDeleteEnum.N.getKey());
            sysPlandefault.eq("hospital_id", -1);
            sysPlandefault.eq(StringUtil.isNotBlank(hospitalPlanVO.getPlanCode()), "plan_code", hospitalPlanVO.getPlanCode());
            sysPlandefault.eq("plan_status", StatusEnum.Enable.getKey());
            Plan sysPlandefaultData = getOne(sysPlandefault, false);
            if (sysPlandefaultData != null) {
                planId = sysPlandefaultData.getId();
                hospitalId = sysPlandefaultData.getHospitalId();
                BeanUtil.copyProperties(sysPlandefaultData, sysPlanInfoDTO);
            }
        }
        HospitalSetVO hospitalSetVO = new HospitalSetVO();
        hospitalSetVO.setHospitalId(hospitalId);
        hospitalSetVO.setPlanId(planId);
        List<PlanDetailDTO> sysSetInfoData = sysSetFacade.getSysSetInfoData(hospitalSetVO);
        sysPlanInfoDTO.setSysSetInfo(sysSetInfoData);
        return sysPlanInfoDTO;
    }

    /**
     * @param hospitalPlanSaveVO
     * @return
     */
    public boolean savePlanInfoDatas(HospitalPlanSaveVO hospitalPlanSaveVO) {
        Date now = DateUtil.now();
        boolean res = false;
        // 校验名称是否相同
        int count = this.count(new QueryWrapper<Plan>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalPlanSaveVO.getHospitalId())
                .eq("plan_name", hospitalPlanSaveVO.getPlanName())
                .eq("plan_code", hospitalPlanSaveVO.getPlanCode())
                .ne("id", hospitalPlanSaveVO.getId() == null ? -1 : hospitalPlanSaveVO.getId()));
        if (count > 0) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "相同【医院】下，方案名称已存在");
        }
        if (hospitalPlanSaveVO.getId() != null) {
            UpdateWrapper<Plan> planQuery = new UpdateWrapper<>();
            planQuery.eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("id", hospitalPlanSaveVO.getId())
                    .set("gmt_modified", now)
                    .set("modifier", SysUserUtils.getCurrentPrincipleID())
                    .set("hospital_id", hospitalPlanSaveVO.getHospitalId())
                    .set("plan_name", hospitalPlanSaveVO.getPlanName())
                    .set("plan_code", hospitalPlanSaveVO.getPlanCode())
                    .set("plan_status", StatusEnum.Enable.getKey());
            res = update(new Plan(), planQuery);
        } else {
            Plan plan = new Plan();
            BeanUtil.copyProperties(hospitalPlanSaveVO, plan);
            plan.setGmtCreate(now);
            plan.setGmtModified(now);
            plan.setCreator(SysUserUtils.getCurrentPrincipleID());
            plan.setModifier(SysUserUtils.getCurrentPrincipleID());
            res = this.save(plan);
        }
        return res;
    }

    public IPage<PlanInfoPageDTO> getPlanInfoPage(HospitalPlanPageVO hospitalPlanPageVO) {
        IPage<PlanInfoPageDTO> data = getPlanInfoPageAll(hospitalPlanPageVO);
        List<PlanInfoPageDTO> planInfos = data.getRecords();
        // 取版本id查明细
        List<Long> ids = new ArrayList<>();
        if (planInfos != null) {
            // 当查出的数据不为空时，取到版本id，再去版本明细表中查询详细的信息
            for (PlanInfoPageDTO planInfo : planInfos) {
                ids.add(planInfo.getId());
            }
            // 获取明细信息
            List<PlanDetailDTO> PlanDetailDatas = sysSetFacade.getByPlanIds(ids);
            Map<Long, List<PlanDetailDTO>> map = new HashMap<>();
            // 获取所有用户开通的产品信息
            map = EntityUtil.makeEntityListMap(PlanDetailDatas, "planId");
            if (map.size() > 0) {
                for (PlanInfoPageDTO planInfo : planInfos) {
                    List<PlanDetailDTO> planDetails = map.get(planInfo.getId());
                    if (ListUtil.isNotEmpty(planDetails)) {
                        planInfo.setSysSetInfo(planDetails);
                    }
                }
            }
        }
        return data.setRecords(planInfos);
    }

    public boolean cancelPlanData(HospitalPlanCancelVO hospitalPlanCancelVO) {
        boolean res = false;
        // 1.先判断数据是否存在有效
        checkPlan(hospitalPlanCancelVO.getPlanId());
        UpdateWrapper<Plan> planNew = new UpdateWrapper<>();
        planNew
                .eq("id", hospitalPlanCancelVO.getPlanId())
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .set("is_deleted", IsDeleteEnum.Y.getKey())
                .set("modifier", UserUtils.getCurrentPrincipleID())
                .set("gmt_modified", DateUtil.now());
        res = update(new Plan(), planNew);

        return res;
    }

    public boolean revStopPlan(PlanRevStopVO planRevStopVO) {
        checkPlan(planRevStopVO.getId());
        UpdateWrapper<Plan> planNew = new UpdateWrapper<>();
        planNew
                .eq("id", planRevStopVO.getId())
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .set("plan_status", planRevStopVO.getStatus())
                .set("modifier", UserUtils.getCurrentPrincipleID())
                .set("gmt_modified", DateUtil.now());
        return update(new Plan(), planNew);
    }

    /**
     * 判断数据是否存在有效
     *
     * @param id
     */
    private Plan checkPlan(Long id) {
        // 1.先判断数据是否存在有效
        QueryWrapper<Plan> PlanFand = new QueryWrapper<>();
        PlanFand.eq("is_deleted", IsDeleteEnum.N.getKey()).eq("id", id);
        Plan plan = getOne(PlanFand, false);
        if (null == plan) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "该数据不存在");
        }
        return plan;
    }


}
